]> Pileus Git - wmpus/blobdiff - wm-wmii.c
Add wm_handle_state functiton
[wmpus] / wm-wmii.c
index 9bf45a93f8a4589896cc774c9a01a47aeec8fba2..409b3e947bc05276a538a3e1b746dbb2a13f5834 100644 (file)
--- a/wm-wmii.c
+++ b/wm-wmii.c
@@ -35,7 +35,7 @@ typedef enum {
 
 typedef enum {
        SPLIT, STACK, FULL, TAB
-} mode_t;
+} layout_t;
 
 typedef enum {
        TILING, FLOATING
@@ -45,44 +45,43 @@ typedef enum {
 struct win_wm { };
 
 typedef struct {
-       win_t  *win;     // the window
-       int     height;  // win height in _this_ tag
-       state_t state;   // state of window
+       win_t   *win;     // the window
+       int      height;  // win height in _this_ tag
 } row_t;
 
 typedef struct {
-       list_t *rows;    // of row_t
-       row_t  *row;     // focused row
-       int     width;   // column width
-       mode_t  mode;    // display mode
+       list_t  *rows;    // of row_t
+       row_t   *row;     // focused row
+       int      width;   // column width
+       layout_t layout;  // column layout
 } col_t;
 
 typedef struct {
-       win_t *win;      // the window
-       int x, y, w, h;  // position of window (in this tag)
-       state_t state;   // state of window
+       win_t   *win;     // the window
+       int x, y, w, h;   // position of window (in this tag)
+       state_t  state;   // state of window
 } flt_t;
 
 typedef struct {
-       list_t *cols;    // of col_t
-       col_t  *col;     // focused col
-       list_t *flts;    // of flt_t
-       flt_t  *flt;     // focused flt
-       layer_t layer;   // focused layer
-       win_t  *geom;    // display size and position
+       list_t  *cols;    // of col_t
+       col_t   *col;     // focused col
+       list_t  *flts;    // of flt_t
+       flt_t   *flt;     // focused flt
+       layer_t  layer;   // focused layer
+       win_t   *geom;    // display size and position
 } dpy_t;
 
 typedef struct {
-       list_t *dpys;    // of dpy_t
-       dpy_t  *dpy;     // focused dpy
-       int     name;    // tag name
+       list_t  *dpys;    // of dpy_t
+       dpy_t   *dpy;     // focused dpy
+       int      name;    // tag name
 } tag_t;
 
 typedef struct {
-       list_t *tags;    // of tag_t
-       tag_t  *tag;     // focused tag
-       win_t  *root;    // root/background window
-       list_t *screens; // display geometry
+       list_t  *tags;    // of tag_t
+       tag_t   *tag;     // focused tag
+       win_t   *root;    // root/background window
+       list_t  *screens; // display geometry
 } wm_t;
 
 #define WIN(node) ((win_t*)(node)->data)
@@ -186,16 +185,16 @@ static int search(tag_t *tag, win_t *target,
        return -1;
 }
 
-/* Set the mode for the windows column in the current tag */
-static void set_mode(win_t *win, mode_t mode)
+/* Set the layout for the windows column in the current tag */
+static void set_mode(win_t *win, layout_t layout)
 {
        col_t *col;
        if (TILING != search(wm_tag, win, NULL, &col, NULL, NULL))
                return;
        printf("set_mode: %p, %d -> %d\n",
-                       col, col->mode, mode);
-       col->mode = mode;
-       if (col->mode == SPLIT)
+                       col, col->layout, layout);
+       col->layout = layout;
+       if (col->layout == SPLIT)
                for (list_t *cur = col->rows; cur; cur = cur->next) {
                        row_t *row = cur->data;
                        row->height = wm_dpy->geom->h;
@@ -269,7 +268,7 @@ static void print_txt(void)
                col_t *col = lcol->data;
                printf("    col:   <%-9p [%p->%p] >%-9p r=%-9p - %dpx @ %d\n",
                                lcol->prev, lcol, lcol->data, lcol->next,
-                               col->row, col->width, col->mode);
+                               col->row, col->width, col->layout);
        for (list_t *lrow = col->rows; lrow; lrow = lrow->next) {
                row_t *row = lrow->data;
                win_t *win = row->win;
@@ -360,6 +359,7 @@ static void put_win_flt(win_t *win, tag_t *tag, dpy_t *dpy)
        flt->h   = dpy->geom->h / 2;
        flt->x   = dpy->geom->x + flt->w / 2;
        flt->y   = dpy->geom->y + flt->h / 2;
+       flt->state = ST_SHOW;
        if (dpy->flt) {
                flt->x = dpy->flt->x + 20;
                flt->y = dpy->flt->y + 20;
@@ -471,7 +471,7 @@ static void shift_focus(int cols, int rows)
                        return;
                row_t *next = get_next(row, rows > 0)->data;
                set_focus(next->win);
-               if (COL(col)->mode != SPLIT)
+               if (COL(col)->layout != SPLIT)
                        wm_update();
        }
        if (cols != 0) {
@@ -636,7 +636,7 @@ static void wm_update_cols(dpy_t *dpy)
                        win->h = ROW(lrow)->height;
                        state_t state = ST_SHOW;
                        int height = 0;
-                       switch (col->mode) {
+                       switch (col->layout) {
                        case SPLIT:
                                sys_move(win, x+margin, y+margin,
                                        col->width, win->h * ((float)my / ty));
@@ -663,7 +663,6 @@ static void wm_update_cols(dpy_t *dpy)
                                        col->width, dpy->geom->h-2*margin);
                                break;
                        }
-                       ROW(lrow)->state = state;
                        sys_show(win, state);
                        if (focus == win)
                                sys_raise(win);
@@ -678,11 +677,17 @@ static void wm_update_cols(dpy_t *dpy)
  *******************************/
 void wm_update(void)
 {
-       /* Show/hide tags */
-       tag_foreach_col(wm_tag, dpy, col, row, win)
-               sys_show(win, ROW(row)->state);
-       tag_foreach_flt(wm_tag, dpy, flt, win)
-               sys_show(win, FLT(flt)->state);
+       /* Updates window sizes */
+       for (list_t *ldpy = wm_tag->dpys; ldpy; ldpy = ldpy->next)
+               wm_update_cols(ldpy->data);
+       tag_foreach_flt(wm_tag, ldpy, lflt, win) {
+               flt_t *flt = lflt->data;
+               sys_move(win, flt->x, flt->y, flt->w, flt->h);
+               sys_raise(flt->win);
+               sys_show(flt->win, flt->state);
+       }
+
+       /* Hide other tags */
        for (list_t *tag = wm ->tags; tag; tag = tag->next)
                if (tag->data != wm_tag) {
                        tag_foreach_col(TAG(tag), dpy, col, row, win)
@@ -691,14 +696,7 @@ void wm_update(void)
                                        sys_show(win, ST_HIDE);
                }
 
-       /* Refresh the display */
-       for (list_t *ldpy = wm_tag->dpys; ldpy; ldpy = ldpy->next)
-               wm_update_cols(ldpy->data);
-       tag_foreach_flt(wm_tag, ldpy, lflt, win) {
-               flt_t *flt = lflt->data;
-               sys_move(win, flt->x, flt->y, flt->w, flt->h);
-               sys_raise(flt->win);
-       }
+       /* Set focused window */
        if (wm_focus)
                set_focus(wm_focus);
 }
@@ -777,7 +775,7 @@ int wm_handle_event(win_t *win, event_t ev, mod_t mod, ptr_t ptr)
                }
        }
 
/* Column mode commands */
        if (mod.MODKEY) {
                switch (ev) {
                case 'd': return set_mode(win, SPLIT), 1;
@@ -855,6 +853,24 @@ int wm_handle_ptr(win_t *cwin, ptr_t ptr)
        return 0;
 }
 
+int wm_handle_state(win_t *win, state_t prev, state_t next)
+{
+       flt_t *flt = NULL;
+
+       if (FLOATING != search(wm_tag, win, NULL, NULL, NULL, &flt)) {
+               cut_win(win, wm_tag);
+               put_win(win, wm_tag, FLOATING);
+               search(wm_tag, win, NULL, NULL, NULL, &flt);
+       }
+
+       flt->state = next;
+
+       if (prev == ST_MAX || prev == ST_FULL)
+               wm_update();
+
+       return 1;
+}
+
 void wm_insert(win_t *win)
 {
        printf("wm_insert: %p\n", win);
@@ -870,7 +886,7 @@ void wm_insert(win_t *win)
 
        /* Arrange */
        wm_update();
-       set_focus(wm_focus);
+       set_focus(win);
        print_txt();
 }
 
@@ -900,7 +916,7 @@ void wm_init(win_t *root)
        wm->tag     = tag_new(wm->screens, 1);
        wm->tags    = list_insert(NULL, wm->tag);
 
-       event_t ev_e[] = {EV_ENTER, EV_FOCUS, EV_MOUSE1};
+       event_t ev_e[] = {EV_ENTER, EV_FOCUS};
        event_t ev_s[] = {'h', 'j', 'k', 'l', 'c', 'q', ' ',
                '0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};
        event_t ev_m[] = {'h', 'j', 'k', 'l', 'd', 's', 'm', 't', ' ',
@@ -777,7 +775,7 @@ int wm_handle_event(win_t *win, event_t ev, mod_t mod, ptr_t ptr)
        }
 
-       /* Column mode commands */
+       /* Column layout commands */
        if (mod.MODKEY) {
                switch (ev) {